@@ -16,26 +16,41 @@ Example usage:
 u'Cabin Fever'
 """
 __author__ = "dbr/Ben"
-__version__ = "1.4"
+__version__ = "1.5"
 
 import os
 import sys
 import urllib
 import urllib2
+import StringIO
 import tempfile
+import warnings
 import logging
+import datetime
 
 try:
     import xml.etree.cElementTree as ElementTree
 except ImportError:
     import xml.etree.ElementTree as ElementTree
 
+try:
+    import gzip
+except ImportError:
+    gzip = None
+
+
 from cache import CacheHandler
 
 from tvdb_ui import BaseUI, ConsoleUI
 from tvdb_exceptions import (tvdb_error, tvdb_userabort, tvdb_shownotfound,
     tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_attributenotfound)
 
+lastTimeout = None
+
+def log():
+    return logging.getLogger("tvdb_api")
+
+
 class ShowContainer(dict):
     """Simple dict that holds a series of Show instances
     """
@@ -73,6 +88,12 @@ class Show(dict):
             # doesn't exist, so attribute error.
             raise tvdb_attributenotfound("Cannot find attribute %s" % (repr(key)))
 
+    def airedOn(self, date):
+        ret = self.search(str(date), 'firstaired')
+        if len(ret) == 0:
+            raise tvdb_episodenotfound("Could not find any episodes that aired on %s" % date)
+        return ret
+
     def search(self, term = None, key = None):
         """
         Search all episodes in show. Can search all data, or a specific key (for
@@ -254,7 +275,8 @@ class Tvdb:
                 custom_ui = None,
                 language = None,
                 search_all_languages = False,
-                apikey = None):
+                apikey = None,
+                forceConnect=False):
         """interactive (True/False):
             When True, uses built-in console UI is used to select the correct show.
             When False, the first search result is used.
@@ -264,8 +286,11 @@ class Tvdb:
             than showing the user a list of more than one series).
             Is overridden by interactive = False, or specifying a custom_ui
 
-        debug (True/False):
-             shows verbose debugging information
+        debug (True/False) DEPRECATED:
+             Replaced with proper use of logging module. To show debug messages:
+
+                 >>> import logging
+                 >>> logging.basicConfig(level = logging.DEBUG)
 
         cache (True/False/str/unicode):
             Retrieved XML are persisted to to disc. If true, stores in tvdb_api
@@ -308,7 +333,20 @@ class Tvdb:
             own key if desired - this is recommended if you are embedding
             tvdb_api in a larger application)
             See http://thetvdb.com/?tab=apiregister to get your own key
+
+        forceConnect (bool):
+            If true it will always try to connect to theTVDB.com even if we
+            recently timed out. By default it will wait one minute before
+            trying again, and any requests within that one minute window will
+            return an exception immediately. 
         """
+        
+        global lastTimeout
+        
+        # if we're given a lastTimeout that is less than 1 min just give up
+        if not forceConnect and lastTimeout != None and datetime.datetime.now() - lastTimeout < datetime.timedelta(minutes=1):
+            raise tvdb_error("We recently timed out, so giving up early this time")
+        
         self.shows = ShowContainer() # Holds all Show classes
         self.corrections = {} # Holds show-name to show_id mapping
 
@@ -348,7 +386,12 @@ class Tvdb:
         self.config['banners_enabled'] = banners
         self.config['actors_enabled'] = actors
 
-        self.log = self._initLogger() # Setups the logger (self.log.debug() etc)
+        if self.config['debug_enabled']:
+            warnings.warn("The debug argument to tvdb_api.__init__ will be removed in the next version. "
+            "To enable debug messages, use the following code before importing: "
+            "import logging; logging.basicConfig(level=logging.DEBUG)")
+            logging.basicConfig(level=logging.DEBUG)
+
 
         # List of language from http://www.thetvdb.com/api/0629B785CE550C8D/languages.xml
         # Hard-coded here as it is realtively static, and saves another HTTP request, as
@@ -382,63 +425,58 @@ class Tvdb:
         self.config['base_url'] = "http://www.thetvdb.com"
 
         if self.config['search_all_languages']:
-            self.config['url_getSeries'] = "%(base_url)s/api/GetSeries.php?seriesname=%%s&language=all" % self.config
+            self.config['url_getSeries'] = u"%(base_url)s/api/GetSeries.php?seriesname=%%s&language=all" % self.config
         else:
-            self.config['url_getSeries'] = "%(base_url)s/api/GetSeries.php?seriesname=%%s&language=%(language)s" % self.config
+            self.config['url_getSeries'] = u"%(base_url)s/api/GetSeries.php?seriesname=%%s&language=%(language)s" % self.config
 
-        self.config['url_epInfo'] = "%(base_url)s/api/%(apikey)s/series/%%s/all/%%s.xml" % self.config
+        self.config['url_epInfo'] = u"%(base_url)s/api/%(apikey)s/series/%%s/all/%%s.xml" % self.config
 
-        self.config['url_seriesInfo'] = "%(base_url)s/api/%(apikey)s/series/%%s/%%s.xml" % self.config
-        self.config['url_actorsInfo'] = "%(base_url)s/api/%(apikey)s/series/%%s/actors.xml" % self.config
+        self.config['url_seriesInfo'] = u"%(base_url)s/api/%(apikey)s/series/%%s/%%s.xml" % self.config
+        self.config['url_actorsInfo'] = u"%(base_url)s/api/%(apikey)s/series/%%s/actors.xml" % self.config
 
-        self.config['url_seriesBanner'] = "%(base_url)s/api/%(apikey)s/series/%%s/banners.xml" % self.config
-        self.config['url_artworkPrefix'] = "%(base_url)s/banners/%%s" % self.config
+        self.config['url_seriesBanner'] = u"%(base_url)s/api/%(apikey)s/series/%%s/banners.xml" % self.config
+        self.config['url_artworkPrefix'] = u"%(base_url)s/banners/%%s" % self.config
 
     #end __init__
 
-    def _initLogger(self):
-        """Setups a logger using the logging module, returns a log object
-        """
-        logger = logging.getLogger("tvdb")
-        formatter = logging.Formatter('%(asctime)s) %(levelname)s %(message)s')
-
-        hdlr = logging.StreamHandler(sys.stdout)
-
-        hdlr.setFormatter(formatter)
-        logger.addHandler(hdlr)
-
-        if self.config['debug_enabled']:
-            logger.setLevel(logging.DEBUG)
-        else:
-            logger.setLevel(logging.WARNING)
-        return logger
-    #end initLogger
-
     def _getTempDir(self):
         """Returns the [system temp dir]/tvdb_api
         """
         return os.path.join(tempfile.gettempdir(), "tvdb_api")
 
     def _loadUrl(self, url, recache = False):
+        global lastTimeout
         try:
-            self.log.debug("Retrieving URL %s" % url)
+            log().debug("Retrieving URL %s" % url)
             resp = self.urlopener.open(url)
             if 'x-local-cache' in resp.headers:
-                self.log.debug("URL %s was cached in %s" % (
+                log().debug("URL %s was cached in %s" % (
                     url,
                     resp.headers['x-local-cache'])
                 )
                 if recache:
-                    self.log.debug("Attempting to recache %s" % url)
+                    log().debug("Attempting to recache %s" % url)
                     resp.recache()
-        except urllib2.URLError, errormsg:
+        except (IOError, urllib2.URLError), errormsg:
+            if not str(errormsg).startswith('HTTP Error'):
+                lastTimeout = datetime.datetime.now()
             raise tvdb_error("Could not connect to server: %s" % (errormsg))
         #end try
-
+        
+        # handle gzipped content,
+        # http://dbr.lighthouseapp.com/projects/13342/tickets/72-gzipped-data-patch
+        if 'gzip' in resp.headers.get("Content-Encoding", ''):
+            if gzip:
+                stream = StringIO.StringIO(resp.read())
+                gz = gzip.GzipFile(fileobj=stream)
+                return gz.read()
+            
+            raise tvdb_error("Received gzip data from thetvdb.com, but could not correctly handle it")
+        
         return resp.read()
 
     def _getetsrc(self, url):
-        """Loads a URL sing caching, returns an ElementTree of the source
+        """Loads a URL using caching, returns an ElementTree of the source
         """
         src = self._loadUrl(url)
         try:
@@ -512,30 +550,31 @@ class Tvdb:
         BaseUI is used to select the first result.
         """
         series = urllib.quote(series.encode("utf-8"))
-        self.log.debug("Searching for show %s" % series)
+        log().debug("Searching for show %s" % series)
         seriesEt = self._getetsrc(self.config['url_getSeries'] % (series))
         allSeries = []
         for series in seriesEt:
             result = dict((k.tag.lower(), k.text) for k in series.getchildren())
+            result['id'] = int(result['id'])
             result['lid'] = self.config['langabbv_to_id'][result['language']]
-            self.log.debug('Found series %(seriesname)s' % result)
+            log().debug('Found series %(seriesname)s' % result)
             allSeries.append(result)
         #end for series
 
         if len(allSeries) == 0:
-            self.log.debug('Series result returned zero')
+            log().debug('Series result returned zero')
             raise tvdb_shownotfound("Show-name search returned zero results (cannot find show on TVDB)")
 
         if self.config['custom_ui'] is not None:
-            self.log.debug("Using custom UI %s" % (repr(self.config['custom_ui'])))
-            ui = self.config['custom_ui'](config = self.config, log = self.log)
+            log().debug("Using custom UI %s" % (repr(self.config['custom_ui'])))
+            ui = self.config['custom_ui'](config = self.config)
         else:
             if not self.config['interactive']:
-                self.log.debug('Auto-selecting first search result using BaseUI')
-                ui = BaseUI(config = self.config, log = self.log)
+                log().debug('Auto-selecting first search result using BaseUI')
+                ui = BaseUI(config = self.config)
             else:
-                self.log.debug('Interactively selecting show using ConsoleUI')
-                ui = ConsoleUI(config = self.config, log = self.log)
+                log().debug('Interactively selecting show using ConsoleUI')
+                ui = ConsoleUI(config = self.config)
             #end if config['interactive]
         #end if custom_ui != None
 
@@ -553,7 +592,7 @@ class Tvdb:
         >>> t['scrubs']['_banners'].keys()
         ['fanart', 'poster', 'series', 'season']
         >>> t['scrubs']['_banners']['poster']['680x1000']['35308']['_bannerpath']
-        'http://www.thetvdb.com/banners/posters/76156-2.jpg'
+        u'http://www.thetvdb.com/banners/posters/76156-2.jpg'
         >>>
 
         Any key starting with an underscore has been processed (not the raw
@@ -561,7 +600,7 @@ class Tvdb:
 
         This interface will be improved in future versions.
         """
-        self.log.debug('Getting season banners for %s' % (sid))
+        log().debug('Getting season banners for %s' % (sid))
         bannersEt = self._getetsrc( self.config['url_seriesBanner'] % (sid) )
         banners = {}
         for cur_banner in bannersEt.findall('Banner'):
@@ -589,7 +628,7 @@ class Tvdb:
             for k, v in banners[btype][btype2][bid].items():
                 if k.endswith("path"):
                     new_key = "_%s" % (k)
-                    self.log.debug("Transforming %s to %s" % (k, new_key))
+                    log().debug("Transforming %s to %s" % (k, new_key))
                     new_url = self.config['url_artworkPrefix'] % (v)
                     banners[btype][btype2][bid][new_key] = new_url
 
@@ -614,12 +653,12 @@ class Tvdb:
         >>> actors[0]['name']
         u'Zach Braff'
         >>> actors[0]['image']
-        'http://www.thetvdb.com/banners/actors/43640.jpg'
+        u'http://www.thetvdb.com/banners/actors/43640.jpg'
 
         Any key starting with an underscore has been processed (not the raw
         data from the XML)
         """
-        self.log.debug("Getting actors for %s" % (sid))
+        log().debug("Getting actors for %s" % (sid))
         actorsEt = self._getetsrc(self.config['url_actorsInfo'] % (sid))
 
         cur_actors = Actors()
@@ -644,10 +683,10 @@ class Tvdb:
         """
 
         if self.config['language'] is None:
-            self.log.debug('Config language is none, using show language')
+            log().debug('Config language is none, using show language')
             getShowInLanguage = language
         else:
-            self.log.debug(
+            log().debug(
                 'Configured language %s override show language of %s' % (
                     self.config['language'],
                     language
@@ -656,7 +695,7 @@ class Tvdb:
             getShowInLanguage = self.config['language']
 
         # Parse show information
-        self.log.debug('Getting all series data for %s' % (sid))
+        log().debug('Getting all series data for %s' % (sid))
         seriesInfoEt = self._getetsrc(
             self.config['url_seriesInfo'] % (sid, getShowInLanguage)
         )
@@ -682,7 +721,7 @@ class Tvdb:
             self._parseActors(sid)
 
         # Parse episode data
-        self.log.debug('Getting all episodes of %s' % (sid))
+        log().debug('Getting all episodes of %s' % (sid))
         epsEt = self._getetsrc( self.config['url_epInfo'] % (sid, language) )
 
         for cur_ep in epsEt.findall("Episode"):
@@ -706,13 +745,13 @@ class Tvdb:
         the correct SID.
         """
         if name in self.corrections:
-            self.log.debug('Correcting %s to %s' % (name, self.corrections[name]) )
+            log().debug('Correcting %s to %s' % (name, self.corrections[name]) )
             sid = self.corrections[name]
         else:
-            self.log.debug('Getting show %s' % (name))
+            log().debug('Getting show %s' % (name))
             selected_series = self._getSeries( name )
             sname, sid = selected_series['seriesname'], selected_series['id']
-            self.log.debug('Got %(seriesname)s, id %(id)s' % selected_series)
+            log().debug('Got %(seriesname)s, id %(id)s' % selected_series)
 
             self.corrections[name] = sid
             self._getShowData(selected_series['id'], selected_series['language'])
@@ -732,7 +771,7 @@ class Tvdb:
         
         key = key.lower() # make key lower case
         sid = self._nameToSid(key)
-        self.log.debug('Got series id %s' % (sid))
+        log().debug('Got series id %s' % (sid))
         return self.shows[sid]
     #end __getitem__
 
@@ -745,7 +784,10 @@ def main():
     """Simple example of using tvdb_api - it just
     grabs an episode name interactively.
     """
-    tvdb_instance = Tvdb(interactive=True, debug=True, cache=False)
+    import logging
+    logging.basicConfig(level=logging.DEBUG)
+
+    tvdb_instance = Tvdb(interactive=True, cache=False)
     print tvdb_instance['Lost']['seriesname']
     print tvdb_instance['Lost'][1][4]['episodename']
 
